/*
  
 Control the uOLED0-160-GDM1 (4dsystems.com.au)
 by hippy (rowanmac @ internode.on.net) July-August 2007

 tested on win32 and linux.
 
 I converted this to C from a C++ ckass I wrote and stripped to only required functionality
 for use with nano-windows.

 *Could* add more functionality like using the display image. 
    
*/


#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "oLED-160-GDM1.h"
// #include "rgb.h"

#ifndef UBYTE
#define UBYTE unsigned char
#endif
#ifndef WORD
#define WORD short
#endif
#ifndef LOBYTE
#define LOBYTE(w)		((UBYTE)(w))
#endif
#ifndef HIBYTE
#define HIBYTE(w)		((UBYTE)(((WORD)(w) >> 8) & 0xFF))
#endif


#ifdef WIN32

#  include <windows.h>
void doSleep(int ms) { Sleep(ms); }
#else
#  include <unistd.h>
#  include <sys/types.h>
#  include <sys/stat.h>
#  include <sys/ioctl.h>
#  include <fcntl.h>
#  include <termios.h>
//#  include <linux/termios.h>


void doSleep(int ms) { sleep(ms); }
#endif

             
#ifdef WIN32
 HANDLE dev;
#else
 int dev;
#endif

/* Close the connection to the device module. */
void oLED_close(void)
{
#ifdef WIN32
    if (dev) CloseHandle(dev);
#else
    if (dev) close(dev);
#endif
    dev = 0;
}

/*
 open a serial port (devicename example "com3", "/dev/ttyUSB0")
 and configure to the maximum baud rate
*/
int oLED_open(const char *devicename)
{
    assert(devicename != NULL);
    oLED_close();

#ifdef WIN32
    DCB dcb;
    dev = CreateFile(devicename, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
    if(GetCommState(dev, &dcb)) // setup port 
    {
     //dcb.BaudRate = CBR_128000;
     dcb.BaudRate = OLED_BAUDRATE;
     dcb.ByteSize = 8;
     dcb.Parity = 0;
     dcb.StopBits = 1;
     dcb.fDtrControl = 0x01;   // allow control of dtr to reset the oLED
     SetCommState(dev, &dcb); 
     
     EscapeCommFunction(dev,5); // reset 
    }
    PurgeComm(dev, PURGE_RXCLEAR|PURGE_TXCLEAR);
    return (dev != NULL);

#else // default to unix
    struct termios params;
    int status;
    int set_bits = TIOCM_DTR;

    dev = open(devicename, O_RDWR);

    if (dev >= 0)
    {
	if (tcgetattr(dev, &params) < 0)
	{
	 fprintf(stderr, "oLED: failed to get serial attributes\n");
	 return 0;
	}
  
    cfmakeraw(&params);
    cfsetspeed(&params,OLED_BAUDRATE);

    ioctl(dev, TIOCMSET, &set_bits); // control the reset pin
    ioctl(dev, TIOCMGET, &status);
   
    if (status & TIOCM_DTR)
     fprintf(stderr,"oLED: DTR is set\n");
    else
     fprintf(stderr,"oLED: DTR is not set\n");

	if (tcsetattr(dev, TCSANOW, &params) < 0) // set new parameters
	{
	 fprintf(stderr,"oLED: failed to set serial attributes\n");
	 return 0;
	}

	tcflush(dev, TCIOFLUSH); // flush input and output buffers
        
	return 1;
    }
    return 0;
#endif
}



/*! Internal. Low level writing of data to the module. Writes \c bytes
  from the buffer \c buf. Returns the number of bytes written. */
int oLED_llWriteData(char *buf, int bytes)
{
    assert(buf);
  //  assert(bytes < 64);

    if (!dev) return 0;

#ifdef WIN32
    unsigned long len;
    WriteFile(dev, buf, bytes, &len, NULL);
    return len;
#else
    int len = write(dev, buf, bytes);
    return len;
#endif
}


/*! Internal. Low level writing of data to the module. Writes \c bytes
  from the buffer \c buf. Returns the number of bytes written. */
int oLED_llWriteDataUnsigned(unsigned char *buf, int bytes)
{
    assert(buf);
   // assert(bytes < 64);

    if (!dev) return 0;

#ifdef WIN32
    unsigned long len;
    WriteFile(dev, buf, bytes, &len, NULL);
    return len;
#else
    int len = write(dev, buf, bytes);
    return len;
#endif
}


/*! Internal. Low level reading of data from the module. Attempts to
  read \c bytes bytes from the module into \c buf. Returns the number
  of bytes read. Is blocking. */
int oLED_llReadData(char *buf, int bytes)
{
    assert(buf);

#ifdef WIN32
    unsigned long r;
    ReadFile(dev, buf, bytes, &r, NULL);
    return r;
#else
    return read(dev, buf, bytes);
#endif
}


/*! Internal. Low level reading of data from the module. Attempts to
  read \c bytes bytes from the module into \c buf. Returns the number
  of bytes read. Is blocking. */
int oLED_llReadDataInt(int *buf, int bytes)
{
    assert(buf);

#ifdef WIN32
    unsigned long r;
    ReadFile(dev, buf, bytes, &r, NULL);
    return r;
#else
    return read(dev, buf, bytes);
#endif
}



/*
  write specific commands directly to the interface
*/  
int oLED_writeCommand(char *buf, int bytes)
{
    assert(buf != NULL);

    int txlen = oLED_llWriteData(buf, bytes);

    return txlen;
}

/* 
   send a SP commands (for display register access)
*/    
int oLED_writeSPCommand(char *buf, int bytes)
{
    assert(buf != NULL);

    char spcmd = OLED_SPCMD; 
    oLED_llWriteData(&spcmd,1);
    int txlen = oLED_llWriteData(buf, bytes);

    return txlen;
}


/*
  write two byte colour info directly to the interface
*/  
int oLED_writeColour(unsigned short clr){
   char xc[] = {HIBYTE(clr),LOBYTE(clr)};
   oLED_llWriteData(&xc,2);      
}






/* 
  read ACK
*/
int oLED_readACK()
{
    if (!dev) return -1;

    char ackbuf;

  //  fprintf(stderr, "expecting ack: ");
    for (;;)
    {
	int c = oLED_llReadData(&ackbuf, 1);
	if (ackbuf != OLED_ACK) 
    {
     fprintf(stderr, "NACK (%d)\n", ackbuf);
     return 0;
    } 
     return 1;
    }
	
}

/* 
  tell the device to detect the baud rate
*/   
int oLED_writeDetectBaud()
{                     
    char cmd = OLED_BAUD_DETECT;
    int x = oLED_llWriteData(&cmd,1);
    return(oLED_readACK());      
}

/* 
  ask the uDisplay for it's version information
*/   
int 
oLED_writeVersionRequest()
{                     
    char cmd = {OLED_VERSION,0x00};  // return version info, do not display
    int x = oLED_llWriteData(&cmd,2);
    return (x == 1);
}

/* 
  ask the uDisplay for it's version information and to show it on screen too.
*/   
int 
oLED_writeVersionRequestOnScreen()
{                     
    char cmd[] = {OLED_VERSION,0x01};  // return version info, display on screen also
    int x = oLED_llWriteData(&cmd,2);
    return (x == 1);
}


int oLED_readVersionRequest()
{
  char val;  

  int x = oLED_llReadData(&val,1);
 // fprintf(stderr, "type: %d\n", val);
  switch (val)
   {
     case 0x00:
      fprintf(stderr,"micro-OLED > ");
      break;
     case 0x01:
      fprintf(stderr,"micro-LCD > ");
      break;
     case 0x02:
      fprintf(stderr,"micro-VGA > ");
      break;
    
     default:
      fprintf(stderr,"unrecognized device type! > ");
   }    
  
  x = oLED_llReadData(&val,1);
  fprintf(stderr, "hardware: %d ", val);
  
  x = oLED_llReadData(&val,1);
  fprintf(stderr, "firmware: %d ", val);
  
  fprintf(stderr, "resolution: ");
  
  x = oLED_llReadData(&val,1);
//  fprintf(stderr, "width: %ld\n", val);
  switch (val)
   {
     case 0x22:
      fprintf(stderr,"220");
      break;
     case 0x28:
      fprintf(stderr,"128");
      break;
     case 0x60:
      fprintf(stderr,"160");
      break;
     case 0x64:
      fprintf(stderr,"64");
      break;
     case 0x76:
      fprintf(stderr,"176");
      break;
     case 0x96:
      fprintf(stderr,"96");
      break;
        
     default:
      fprintf(stderr,"???");
   } 
  
  fprintf(stderr,"x");
  
  x = oLED_llReadData(&val,1);
  //fprintf(stderr, "height: %ld\n", val);

  switch (val)
   {
     case 0x22:
      fprintf(stderr,"220");
      break;
     case 0x28:
      fprintf(stderr,"128");
      break;
     case 0x60:
      fprintf(stderr,"160");
      break;
     case 0x64:
      fprintf(stderr,"64");
      break;
     case 0x76:
      fprintf(stderr,"176");
      break;
     case 0x96:
      fprintf(stderr,"96");
      break;
        
     default:
      fprintf(stderr,"???");
   } 
    
    fprintf(stderr,"\n");
    
    return 0;
}


// set a pixel
int oLED_SetPixel(char x, char y, unsigned short clr){
   char cmd2[] = {OLED_PIXEL, x, y, HIBYTE(clr), LOBYTE(clr)};
   oLED_llWriteData(&cmd2,5);
   return(oLED_readACK());
}


// get a pixel
unsigned short oLED_GetPixel(char x, char y){
   unsigned short c;
   char cmd2[] = {OLED_READ_PIXEL,x,y};
   oLED_llWriteData(&cmd2,3); // ask for colour
   oLED_llReadData(&c,2); // receive colour
   
   return c;
}



// set the background colour
int oLED_SetBackColor(unsigned short clr){  
   char cmd = OLED_BACKGROUND;
   oLED_llWriteData(&cmd,1);
   oLED_writeColour(clr);
   return(oLED_readACK());
}

// erase the display
int oLED_EraseScreen(void){
   char cmd = OLED_ERASE;
   oLED_llWriteData(&cmd,1);
   return(oLED_readACK());
}


int oLED_DrawLine(char x1, char y1, char x2, char y2, unsigned short clr){
   char cmd2[] = {OLED_LINE,x1,y1,x2,y2,HIBYTE(clr),LOBYTE(clr)};
   oLED_llWriteData(&cmd2,7);
   return(oLED_readACK());
}   




int oLED_DrawRectangle(char x1, char y1, char x2, char y2, unsigned short clr){
   char cmd2[] = {OLED_RECTANGLE, x1, y1, x2, y2, HIBYTE(clr), LOBYTE(clr)};
   oLED_llWriteData(&cmd2,7);
   return(oLED_readACK());
}   



int oLED_DrawString(char x, char y, char fontsize, unsigned short clr, char *buf, int bytes ){
   char cmd[] = {OLED_STRING_FORMAT,x,y,fontsize};
   oLED_llWriteData(cmd,4);
   oLED_writeColour(clr);
   oLED_llWriteData(buf,bytes);   
   return(oLED_readACK());
}   


// copy block src to dest of width/height
int oLED_BlkCopy(char src_x, char src_y, char dest_x, char dest_y, char width, char height){
   char cmd[] = {OLED_BLK_COPY, src_x, src_y, dest_x, dest_y, width, height};

   oLED_llWriteData(&cmd,7);
   return(oLED_readACK());
}


/*
// Set text to Opaque(1)/Transparent(0)
int oLED::FontMode(char fntmode){
 if  (dev == 0) {return 0;}  
   char cmd = OLED_FONT_OPAQUE;
   llWriteData(&cmd,1);
   cmd = fntmode;
   llWriteData(&cmd,1);
   return(readACK());
}
*/

// Set solid fill (0) or wireframe (1)
int oLED_PenSize(char pensize){
 if  (dev == 0) {return 0;}  
   char cmd = OLED_PEN_SIZE;
   oLED_llWriteData(&cmd,1);
   cmd = pensize;
   oLED_llWriteData(&cmd,1);
   return(oLED_readACK());
}



// Power Down Display
int oLED_PowerDown(void){
 if  (dev == 0) {return 0;}  
   char cmd = OLED_DISPLAY_CTRL;
   oLED_llWriteData(&cmd,1);
   cmd = OLED_CTRL_POWER;
   oLED_llWriteData(&cmd,1);
   cmd = OLED_POWER_DOWN;
   oLED_llWriteData(&cmd,1);
   return(oLED_readACK());
}






// Usage: oLED_init <device> 
int oLED_init(char *devicename)
{
   
  if (oLED_open(devicename))
   {

    fprintf(stderr, "oLED: init()");
 
    oLED_writeDetectBaud();
    
    oLED_SetBackColor(0);
    oLED_EraseScreen();

    return 1;
   }       
    else
    {
	fprintf(stderr, "oLED: failed to open device\n");
	return 0;
    }
   
}

 
